(* Copy files via FTP

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Oct. 2008
   *)

unit FtpCopyMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, NumberEd, ComCtrls, Buttons, HListBox, FtpDlg,
  TransferStat, FtGlobal, ExtFileTools, FtpFileTools, WinApiUtils, XPMan;


const
  ProgName = 'FTP-Copy';
  Vers = ' - Vers. 3.2';
  CopRgt = ' 2009 - Dr. J. Rathlev - IEAP-Kiel';
  EmailAdr = 'info(a)rathlev-home.de';

  MaxUserPwdLength = 64;

  // SSL-Zertifikate
  RootCertFile = 'rootcertfile.pem';
  CertFile     = 'certfile.crt';
  KeyFile      = 'keyfile.key';
  SSLPassword  = 'Password';

// Schlssel zum Kodieren der Passwrter in der Ini-Datei
  PwdKey = '8AA3D2D60655F3581324FC174A05A5E79927CCBD358641723D'; //Delphi7JR

type
  TCopySettings = class(TObject)
    TaskName,
    Source      : string;
    FtpPar      : TFtpParams;
    SFiles      : TStringList;
    CheckTimeStamp,
    Rename      : boolean;
    NewName     : string;
    constructor Create(AName : string);
    destructor Destroy; override;
    end;

  TMainForm = class(TForm)
    Label1: TLabel;
    btnAddFiles: TSpeedButton;
    btnCopy: TBitBtn;
    btnQuit: TBitBtn;
    btnInfo: TBitBtn;
    btnFtp: TBitBtn;
    XPManifest1: TXPManifest;
    Label3: TLabel;
    edDest: TEdit;
    btnCopyName: TSpeedButton;
    cbRename: TCheckBox;
    Label2: TLabel;
    lbFiles: TListBox;
    btnNewtask: TSpeedButton;
    cbxSource: TComboBox;
    edNewName: TEdit;
    btnDelTask: TBitBtn;
    cbOverwrite: TCheckBox;
    btnRemFiles: TSpeedButton;
    btnNewFiles: TSpeedButton;
    laStatus: TLabel;
    cbxTask: TComboBox;
    Label4: TLabel;
    btnSourceDir: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure btnInfoClick(Sender: TObject);
    procedure btnQuitClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnAddFilesClick(Sender: TObject);
    procedure btnFtpClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure btnCopyNameClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SourceSelectBtnClick(Sender: TObject);
    procedure cbxTaskCloseUp(Sender: TObject);
    procedure btnDelTaskClick(Sender: TObject);
    procedure btnRemFilesClick(Sender: TObject);
    procedure btnNewFilesClick(Sender: TObject);
    procedure cbRenameClick(Sender: TObject);
    procedure edNewNameExit(Sender: TObject);
    procedure btnNewtaskClick(Sender: TObject);
    procedure cbOverwriteClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    UserPath,
    AppPath,IniName  : string;
    FTP              : TExtFTP;
    CopyThread       : TCopyThread;
    Force,
    Connecting       : boolean;
    FSize            : int64;
    DefCopySettings,
    CopySettings     : TCopySettings;
    procedure ShowData(ACopySettings : TCopySettings);
    procedure CancelCopy(var Msg: TMessage); message wmCancelAction;
    procedure ShowProgress (AAction : TFileAction; ACount: int64);
    procedure StartCopy (ACopySettings : TCopySettings);  public
    { Public-Deklarationen }
  public
    procedure CheckRemote;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses JrUtils, WinDesktop, ShlObj, IniFiles, Crypt, PwdDlg, StrUtils,
   IdFTPList, IdFtp, HttpApp, ShellDirDlg, GnuGetText, GnuGetTextInit,
   OpenFilesDlg;

{ ------------------------------------------------------------------- }
constructor TCopySettings.Create(AName : string);
begin
  inherited Create;
  SFiles:=TStringList.Create;
  TaskName:=AName;
  Source:=''; NewName:=''; Rename:=false; CheckTimeStamp:=true;
  FtpPar:=FtpDialog.DefaultValues;
  end;

destructor TCopySettings.Destroy;
begin
  SFiles.Free;
  inherited Destroy;
  end;

{ ------------------------------------------------------------------- }
const
  IniExt = 'ini';

  CfGSekt    = 'Config';
  TaskSekt   = 'Task';
  SourceSekt = 'Source';

  iniTName  = 'TaskName';
  iniLast   = 'LastTask';
  iniSCount = 'TaskCount';
  iniSDir   = 'SourceDir';
  iniFCount = 'FileCount';
  iniSFile  = 'File';
  iniPort   = 'Port';
  iniHost   = 'Host';
  iniDir    = 'Directory';
  iniUser   = 'Username';
  iniPwd    = 'Password';
  iniPassive = 'Passive';
  iniCMode   = 'ChangeNames';
  IniPrxServ = 'ProxyServer';
  IniPrxPort = 'ProxyPort';
  IniPrxUser = 'ProxyUsername';
  IniPrxPwd  = 'ProxyPassword';
  IniPrxType = 'ProxyType';
  IniTimeSt  = 'CheckTimeStamp';
  IniRename  = 'Rename';
  iniDest    = 'DestFile';

procedure TMainForm.FormCreate(Sender: TObject);
var
  IniFile : TIniFile;
  ndx,
  i,j,m,n : integer;
  sn,
  sekt,s  : string;
  ts      : TCopySettings;
begin
  TranslateComponent(self);
  InitPaths(AppPath,UserPath);
  InitVersion(ProgName,Vers,CopRgt,ProgVersName,ProgVersDate);
  Caption:=_('Copy file(s) via FTP')+VersInfo.Comments;
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  sn:=''; Force:=false;
  if ParamCount>0 then for i:=1 to ParamCount do begin
    s:=ParamStr(i);
    if s[1]='/' then begin
      Delete(s,1,1);
      if CompareOption(s,'force') then Force:=true;
      if ReadOptionValue(s,'ini') then IniName:=s;
      end
    else sn:=s;
    end;
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    n:=ReadInteger(CfgSekt,iniSCount,0);
    for i:=0 to n-1 do begin
      sekt:=TaskSekt+ZStrint(i,2);
      s:=ReadString(sekt,iniTName,'');
      if length(s)>0 then begin
        ts:=TCopySettings.Create(s);
        with ts do begin
          Source:=ReadString(sekt,iniSDir,'');
          m:=ReadInteger(Sekt,iniFCount,0);
          for j:=0 to m-1 do begin
            s:=ReadString(sekt,iniSFile+ZStrint(j,3),'');
            if length(s)>0 then SFiles.Add(s);
            end;
          with FtpPar do begin
            Host:=ReadString(Sekt,iniHost,'');
            Port:=ReadInteger(Sekt,iniPort,defPort);
            Username:=ReadString(Sekt,iniUser,'');
            Password:=DecryptPwdString(DecodeString(PwdKey),ReadString(Sekt,iniPwd,''),teAscii85);
            Directory:=ReadString(Sekt,iniDir,'');
            Passive:=ReadBool(Sekt,iniPassive,false);
            CaseMode:=TTextChange(ReadInteger (Sekt,iniCMode,integer(tcNone)));
            if (length(Host)>0) and (length(Username)>0) then
            with Proxy do begin
              Server:=ReadString(Sekt,iniPrxServ,'');
              Port:=ReadInteger(Sekt,iniPrxPort,defProxyPort);
              Username:=ReadString(Sekt,iniPrxUser,'');
              Password:=DecryptPwdString(DecodeString(PwdKey),ReadString(Sekt,IniPrxPwd,''),teAscii85);
              Mode:=TIdFtpProxyType(ReadInteger(Sekt,IniPrxType,0));
              end;
            end;
          CheckTimeStamp:=ReadBool(Sekt,iniTimeSt,true);
          Rename:=ReadBool(Sekt,iniRename,false);
          NewName:=ReadString(Sekt,IniDest,'');
          end;
        cbxTask.AddItem(ts.TaskName,ts);
        end;
      end;
    ndx:=ReadInteger(CfgSekt,iniLast,0);
    LoadHistory(IniFile,SourceSekt,'',cbxSource.Items);
    Free;
    end;
  if length(sn)>0 then ndx:=cbxTask.Items.IndexOf(sn);
  cbxTask.ItemIndex:=ndx;
  DefCopySettings:=TCopySettings.Create('');
  CopySettings:=DefCopySettings;
  Connecting:=false;
  end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  DefCopySettings.Free;
  end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  IniFile : TIniFile;
  i,j     : integer;
  sekt    : string;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    with cbxTask,Items do begin
      WriteInteger(CfgSekt,iniSCount,Count);
      WriteInteger(CfgSekt,iniLast,ItemIndex);
      for i:=0 to Count-1 do with objects[i] as TCopySettings do begin
        sekt:=TaskSekt+ZStrint(i,2);
        EraseSection(Sekt);
        WriteString(sekt,iniTName,TaskName);
        WriteString(sekt,iniSDir,Source);
        WriteInteger(Sekt,iniFCount,SFiles.Count);
        with SFiles do for j:=0 to Count-1 do begin
          WriteString(sekt,iniSFile+ZStrint(j,3),Strings[j]);
          end;
        with FtpPar do begin
          WriteString(Sekt,iniHost,Host);
          WriteInteger(Sekt,iniPort,Port);
          WriteString(Sekt,iniUser,Username);
          WriteString(Sekt,iniPwd,EncryptPwdString(DecodeString(PwdKey),Password,MaxUserPwdLength,teAscii85));
          WriteString(Sekt,iniDir,Directory);
          WriteBool(Sekt,iniPassive,Passive);
          WriteInteger(Sekt,iniCMode,integer(CaseMode));
          with Proxy do begin
            WriteString(Sekt,iniPrxServ,Server);
            WriteInteger(Sekt,iniPrxPort,Port);
            WriteString(Sekt,iniPrxUser,Username);
            WriteString(Sekt,iniPrxPwd,EncryptPwdString(DecodeString(PwdKey),Password,MaxUserPwdLength,teAscii85));
            WriteInteger(Sekt,IniPrxType,integer(Mode));
            end;
          end;
        WriteBool(Sekt,iniTimeSt,CheckTimeStamp);
        WriteBool(Sekt,iniRename,Rename);
        WriteString(Sekt,IniDest,NewName);
        SaveHistory(IniFile,SourceSekt,'',true,cbxSource.Items);
        Free;
        end;
      end;
    Free;
    end;
  end;

procedure TMainForm.CheckRemote;
begin
  with cbxTask do if Force and (Items.Count>0) and (ItemIndex>=0) then begin
//    InfoDialog('',IniName+sLineBreak+ts.taskname);
    StartCopy(Items.Objects[ItemIndex] as TCopySettings);
    Application.Terminate;
    end;
  end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  FtpDialog.Init(Ininame,'');
  with cbxTask do if (Items.Count>0) and (ItemIndex>=0) then
      CopySettings:=Items.objects[ItemIndex] as TCopySettings;
  ShowData(CopySettings);
  end;

procedure TMainForm.ShowData(ACopySettings : TCopySettings);
var
  i : integer;
begin
  with ACopySettings do begin
    AddToHistory(cbxSource.Items,Source);
    cbxSource.ItemIndex:=0;
    lbFiles.Clear;
    with SFiles do begin
      for i:=0 to Count-1 do lbFiles.Items.Add(Strings[i]);
      cbRename.Enabled:=Count=1;
      end;
    btnAddFiles.Enabled:=SFiles.Count>0;
    btnremFiles.Enabled:=btnAddFiles.Enabled;
    with FtpPar do if length(Host)>0 then
      edDest.Text:='ftp://'+Host+':'+IntToStr(Port)+'/'+Directory
    else edDest.Text:='';
    btnCopy.Enabled:=(length(edDest.Text)>0) and (lbFiles.Items.Count>0);
    cbOverwrite.Checked:=CheckTimeStamp;
    if cbRename.Enabled then begin
      cbRename.Checked:=Rename;
      edNewName.Text:=NewName;
      end
    else begin
      cbRename.Checked:=false;
      edNewName.Text:='';
      end;
    edNewName.Enabled:=cbRename.Checked;
    btnCopyName.Enabled:=cbRename.Checked;
    end;
  end;

procedure TMainForm.cbOverwriteClick(Sender: TObject);
begin
  CopySettings.CheckTimeStamp:=cbOverwrite.Checked;
  end;

procedure TMainForm.cbRenameClick(Sender: TObject);
begin
  CopySettings.Rename:=cbRename.Checked;
  edNewName.Enabled:=cbRename.Checked;
  btnCopyName.Enabled:=cbRename.Checked;
  end;

procedure TMainForm.edNewNameExit(Sender: TObject);
begin
  CopySettings.NewName:=edNewName.Text;
  end;

procedure TMainForm.btnInfoClick(Sender: TObject);
begin
  InfoDialog(ProgName,ProgVersName+' - '+ProgVersDate+#13+
           VersInfo.CopyRight+#13'D-24098 Kiel',CenterPos);
  end;

procedure TMainForm.btnQuitClick(Sender: TObject);
begin
  Close;
  end;

procedure TMainForm.btnNewtaskClick(Sender: TObject);
var
   s : string;
begin
  s:='';
  if InputQuery(_('New copy task'),_('Taskname:'),s) then begin
    CopySettings:=TCopySettings.Create(s);
    with cbxTask do ItemIndex:=Items.AddObject(s,CopySettings);
    SourceSelectBtnClick(Sender);
    end;
  end;

procedure TMainForm.SourceSelectBtnClick(Sender: TObject);
var
  s : string;
begin
  if length(cbxSource.Text)>0 then s:=cbxSource.Text
  else s:=UserPath;
  if ShellDirDialog.Execute(_('Select source directory'),true,true,UserPath,s) then begin
    CopySettings.Source:=IncludeTrailingPathDelimiter(s);
    ShowData(CopySettings);
    end;
  end;

procedure TMainForm.cbxTaskCloseUp(Sender: TObject);
begin
  with cbxTask do if Items.Count>0 then begin
    CopySettings:=Items.Objects[ItemIndex] as TCopySettings;
    ShowData(CopySettings);
    end;
  end;

procedure TMainForm.btnNewFilesClick(Sender: TObject);
var
  i   : integer;
  fl  : TStringList;
begin
  if (length(cbxSource.Text)>0) then with CopySettings do begin
    if (SFiles.Count=0) or ConfirmDialog('',_('Replace all files?')) then begin
      fl:=TStringList.Create;
      if OpenFilesDialog.Execute(_('File(s) to copy'),cbxSource.Text,
                 _('all|*.*|Pdf|*.pdf|Html|*.html|Zip|*.zip|Exe|*.exe'),fl) then begin
        SFiles.Clear;
        with fl do for i:=0 to Count-1 do SFiles.Add(Strings[i]);
        ShowData(CopySettings);
        end;
      fl.Free;
      end;
    end
  else ErrorDialog('',_('Select directory first!'));
  end;

procedure TMainForm.btnAddFilesClick(Sender: TObject);
var
  i   : integer;
  fl  : TStringList;
begin
  if (length(cbxSource.Text)>0) then with CopySettings do begin
    fl:=TStringList.Create;
    if OpenFilesDialog.Execute(_('File(s) to copy'),cbxSource.Text,
                 _('all|*.*|Pdf|*.pdf|Html|*.html|Zip|*.zip|Exe|*.exe'),fl) then begin
      with fl do for i:=0 to Count-1 do SFiles.Add(Strings[i]);
      ShowData(CopySettings);
      end;
    fl.Free;
    end
  else ErrorDialog('',_('Select directory first!'));
  end;

procedure TMainForm.btnRemFilesClick(Sender: TObject);
var
  i,n : integer;
begin
  if ConfirmDialog('',Format(_('Remove %u file(s)?'),[lbFiles.SelCount])) then
      with CopySettings do begin
    with lbFiles do for i:=Items.Count-1 downto 0 do if Selected[i] then begin
      n:=SFiles.IndexOf(Items[i]);
      if n>=0 then SFiles.Delete(n);
      end;
    end;
  ShowData(CopySettings);
  end;

procedure TMainForm.btnCopyNameClick(Sender: TObject);
begin
  edNewName.Text:=CopySettings.SFiles[0];
  cbRename.Checked:=true;
  end;

procedure TMainForm.btnFtpClick(Sender: TObject);
begin
  with CopySettings do if FtpDialog.Execute (FtpPar,false,false) then begin
    with FtpPar do edDest.Text:='ftp://'+Host+':'+IntToStr(Port)+'/'+Directory;
    ShowData(CopySettings);
    end;
  end;

procedure TMainForm.btnDelTaskClick(Sender: TObject);
var
  n : integer;
begin
  with cbxTask do if (Items.Count>0) and
      ConfirmDialog('',Format(_('Remove task: '+sLineBreak+'%s?'),[Text])) then begin
    n:=ItemIndex;
    CopySettings.Free;
    Items.Delete(ItemIndex);
    if Items.Count=0 then CopySettings:=DefCopySettings
    else begin
      if n<Items.Count then ItemIndex:=n else ItemIndex:=Items.Count-1;
      CopySettings:=(Items.Objects[ItemIndex] as TCopySettings);
      end;
    ShowData(CopySettings);
    end;
  end;

{ ------------------------------------------------------------------- }
(* Kopierfortschritt aktualisieren *)
procedure TMainForm.ShowProgress (AAction : TFileAction; ACount : int64);
begin
  if ACount<0 then begin
    FSize:=abs(ACount);
    TransferStatus.UpdateProgress(0,FSize);
    end
  else TransferStatus.UpdateProgress(ACount,FSize);
  end;

(* Nachricht von Fortschrittsanzeige bei Klick auf Abbrechen-Button *)
procedure TMainForm.CancelCopy(var Msg: TMessage);
begin
  if assigned(CopyThread) then with CopyThread do if not Done then CancelThread
  else if Connecting then FTP.Disconnect;
  end;

procedure TMainForm.btnCopyClick(Sender: TObject);
begin
  with CopySettings do begin
    NewName:=edNewName.Text;
    Rename:=cbRename.Checked;
    CheckTimeStamp:=cbOverwrite.Checked;
    with FtpPar do begin
      if (length(Host)=0) or (length(Username)=0) then btnFtpClick(self);
      end;
    end;
  StartCopy(CopySettings);
  end;


procedure TMainForm.StartCopy (ACopySettings : TCopySettings);
var
  ok  : boolean;
  s,t : string;
  i,n : integer;
begin
  with ACopySettings do if SFiles.Count>0 then begin
    laStatus.Caption:='';
    with FtpPar do if (length(Host)=0) or (length(Username)=0) then begin
      ErrorDialog('',_('No FTP host or username'));
      Exit;
      end;
    FTP:=TExtFtp.Create(self);
    with FtpPar do begin
//      Ftp.SetCertificate(RootCertFile,CertFile,KeyFile,SSLPassword);
      Ftp.Host:=Host;
      Ftp.Port:=Port;
      Ftp.Username:=Username;
      FTP.SecureTransfer:=SecureMode;
      Ftp.Passive:=Passive;
      Ftp.CaseMode:=CaseMode;
      Ftp.TransferTimeout:=defTimeout;
      if length(Password)=0 then begin
        ok:=PasswordDialog.Execute (_('FTP connection'),
            Format(_('The connection to server:'+sLineBreak+'  %s:%u'+
              sLineBreak+'for user: "%s" requires a password!'),
              [Host,Port,Username]),Password,false)=mrOK;
        end
      else ok:=true;
      if ok and (Proxy.Mode<>fpcmNone) then begin // proxy settings
        Ftp.ProxySettings.ProxyType:=Proxy.Mode;
        Ftp.ProxySettings.Host:=Proxy.Server;
        Ftp.ProxySettings.Port:=Proxy.Port;
        Ftp.ProxySettings.Username:=Proxy.Username;
        with Proxy do if length(Password)=0 then
          ok:=PasswordDialog.Execute (rsFtpConn,
              Format(rsProxyHint,[Server,Port,Username]),Password,false)=mrOK;
        end;
      if ok then begin
        TransferStatus.ShowWindow(Handle,_('Transfer files'),
                     'ftp://'+Host+':'+IntToStr(Port),_('Connecting to server'));
        Application.ProcessMessages;
        with Ftp.ProxySettings do if ProxyType<>fpcmNone then Password:=Proxy.Password;
        Ftp.Password:=Password;
        try
          Connecting:=true;
          Ftp.Connect;
        except
          ok:=false;
          end;
        Application.ProcessMessages;
        Connecting:=false;
        if ok then ok:=not TransferStatus.Stopped and Ftp.ForceDir(Directory);
        if ok then Ftp.GetFileList;
        end;
      end;
    if ok then begin
      Application.ProcessMessages;
      n:=0;
      with TransferStatus do begin
        Directory:='/'+FtpPar.Directory;
        StartCopy;
        end;
      with SFiles do for i:=0 to Count-1 do begin
        if TransferStatus.Stopped then Break;
        if (i=0) and Rename and (length(NewName)>0) then begin
          s:=DosPathToUnixPath(ExcludeTrailingPathDelimiter(ExtractFilePath(NewName)));
          if Ftp.ForceDir(s) then s:=ExtractFileName(NewName)
          else begin
            TransferStatus.Close;
            ErrorDialog('',Format(_('Cannot create remote directory:'+sLineBreak+'%s on %s!'),
                           [FtpPar.Host,s]),CenterPos);
            exit;
            end;
          end
        else begin
          s:=Strings[i];
          end;
        t:=Source+Strings[i];
        FSize:=LongFileSize(Source+Strings[i]);
        if CheckTimeStamp then
          ok:=Ftp.CheckTimeStamp(s,FileDateToDateTime(FileAge(t)),2)
//          ok:=Ftp.CheckTimeStamp(s,FileTimeToDateTime(GetFileLastWriteTime(t)),2)
        else ok:=true;
        if ok then begin
          with TransferStatus do begin
            Position:=0;
            if Rename then Status:=Strings[i]+_(' to ')+s
            else Status:=Strings[i];
            end;
          Application.ProcessMessages;
          CopyThread:=TWriteFtpThread.Create(Ftp,t,s,nil,false,false,defFtpBufferSize,tpNormal);
          with CopyThread do begin
            OnProgress:=ShowProgress;
            repeat
              Sleep(10);
              Application.ProcessMessages;
              until Done;
            if ErrorCode<>errOK then TransferStatus.Status:='==> '+GetCopyErrMsg(ErrorCode);
            inc(n);
            Free;
            end;
          Ftp.SetTimeStamp(t,s);
          end;
        end;
      TransferStatus.CloseWindow(0,n);
      laStatus.Caption:=Format(_('%u File(s) copied'),[n]);
      FTP.Disconnect;
      end
    else begin
      TransferStatus.Close;
      ErrorDialog('',Format(_('Connection to FTP server:'+sLineBreak+'%s failed!'),[FtpPar.Host]),CenterPos);
      end;
    Ftp.Free;
    end
  else ErrorDialog('',_('No files selected!'));;
  end;

end.
