(* Delphi Dialog
   Dialog: Eingabe der FTP-Verbindungsparameter
   ============================================

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Dez. 2006
   Vers. 2 - Jul. 2007
   Vers. 3 - Aug. 2008   
   *)

unit FtpDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, NumberEd, HListBox, FtpFileTools;

const
  defPort = 21;
  defProxyPort = 2121;

type
  TFtpDialog = class(TForm)
    CancelBtn: TBitBtn;
    OKBtn: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    hcbServer: THistoryCombo;
    hcbUser: THistoryCombo;
    hcbDir: THistoryCombo;
    edtFtpPwd: TEdit;
    Label5: TLabel;
    edtPort: TRangeEdit;
    btnShow: TSpeedButton;
    cbxCase: TComboBox;
    Label6: TLabel;
    cbPassive: TCheckBox;
    cbProxy: TCheckBox;
    edProxy: TEdit;
    btProxy: TBitBtn;
    cbSecure: TCheckBox;
    procedure btnShowClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbProxyClick(Sender: TObject);
    procedure btProxyClick(Sender: TObject);
    procedure cbSecureClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private-Deklarationen }
    FIniName : string;
    sp    : boolean;
    FSecureMode : integer;
    FProxyPar   : TFtpProxyParams;
    procedure SetPassiv;
    procedure ShowProxy;
  public
    { Public-Deklarationen }
    procedure Init (AIniName,DefIni : string);
    function DefaultValues : TFtpParams;
    function Execute (var AFtpPar : TFtpParams; ShowPwd,ShowSecure : boolean) : boolean;
  end;

function GetFtpHint(FtpPar : TFtpParams) : string;

var
  FtpDialog: TFtpDialog;

implementation

{$R *.dfm}

uses IniFiles, Idftp, JrUtils, GnuGetText, FtpProxyDlg;

{ ------------------------------------------------------------------- }
function GetFtpHint(FtpPar : TFtpParams) : string;
begin
  with FtpPar do begin
    if SecureMode>0 then Result:='ftps' else Result:='ftp';
    Result:=Result+'://'+Host+DPunkt+IntToStr(Port)+'/'+Directory;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TFtpDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  FIniName:=''; sp:=true;
  end;

const
  ServerSekt = 'FtpServer';
  UserSekt   = 'Usernames';
  DirSekt    = 'FtpDirs';

procedure TFtpDialog.Init (AIniName,DefIni : string);
var
  si      : string;
begin
  // nur beim Neustart laden
  if (length(FIniName)=0) and ((length(AIniName)>0) or (length(DefIni)>0)) then begin
    FIniName:=AIniName;
    if FileExists(FIniName) then si:=FIniName else si:=DefIni;
    if FileExists(si) then begin
      hcbServer.LoadFromIni(si,ServerSekt);
      hcbUser.LoadFromIni(si,UserSekt);
      hcbDir.LoadFromIni(si,DirSekt);
      end;
    end;
  end;

function TFtpDialog.DefaultValues : TFtpParams;
begin
  with Result do begin
    Host:='';
    Port:=defPort;
    Username:='';
    Password:='';
    Directory:='';
    SecureMode:=0;
    Passive:=false;
    CaseMode:=tcNone;
    with Proxy do begin
      Server:='';
      Username:='';
      Password:='';
      Port:=defProxyPort;
      Mode:=fpcmNone;
      end;
    end;
  end;

procedure TFtpDialog.btnShowClick(Sender: TObject);
begin
  with edtFtpPwd do if btnShow.Down then PasswordChar:=#0 else PasswordChar:='*';
  end;

procedure TFtpDialog.ShowProxy;
begin
  with FProxyPar do begin
    if cbProxy.Checked and (length(Server)>0) then edProxy.Text:=Server+':'+IntToStr(Port)
    else edProxy.Text:='';
    edProxy.Enabled:=cbProxy.Checked;
    btProxy.Enabled:=cbProxy.Checked;
    end;
  end;

procedure TFtpDialog.cbProxyClick(Sender: TObject);
begin
  if Visible and cbProxy.Checked then begin
    FtpProxyDialog.Execute (FProxyPar,sp);
    cbProxy.Checked:=FProxyPar.Mode<>fpcmNone;
    end;
  ShowProxy;
  end;

procedure TFtpDialog.btProxyClick(Sender: TObject);
begin
  FtpProxyDialog.Execute (FProxyPar,sp);
  cbProxy.Checked:=FProxyPar.Mode<>fpcmNone;
  ShowProxy;
  end;

procedure TFtpDialog.SetPassiv;
begin
  with cbPassive do begin
    Enabled:=fSecureMode=0;
    Checked:=true;            // muss auf "passiv" stehen
    end;
  end;

procedure TFtpDialog.cbSecureClick(Sender: TObject);
begin
  if cbSecure.Checked then fSecureMode:=2
  else fSecureMode:=0;
  SetPassiv;
  end;

procedure TFtpDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult=mrOK then begin
    CanClose:=(length(hcbServer.Text)>0) or (length(hcbUser.Text)>0);
    if not Canclose then begin
      if length(hcbServer.Text)=0 then begin
        ErrorDialog('',_('No host specified!'));
        hcbServer.SetFocus;
        end
      else begin
        ErrorDialog('',_('No username specified!'));
        hcbUser.SetFocus;
        end;
      end;
    end;
  end;

function TFtpDialog.Execute (var AFtpPar : TFtpParams; ShowPwd,ShowSecure : boolean) : boolean;
begin
  sp:=ShowPwd;
  btnShow.Visible:=ShowPwd;
  cbSecure.Visible:=ShowSecure;
  with AFtpPar do begin
    hcbServer.Text:=Host;
    edtPort.Value:=Port;
    hcbUser.Text:=Username;
    edtFtpPwd.Text:=Password;
    hcbDir.Text:=Directory;
    FSecureMode:=SecureMode;
    cbSecure.Checked:=SecureMode>0;
    with cbPassive do if SecureMode>0 then begin
      Enabled:=false;
      Checked:=true;
      end
    else begin
      Enabled:=true;
      Checked:=Passive;
      end;
    cbxCase.ItemIndex:=integer(CaseMode);
    FProxyPar:=Proxy;
    cbProxy.Checked:=FProxyPar.Mode<>fpcmNone;
    ShowProxy;
    if ShowModal=mrOK then begin
      Host:=hcbServer.Text;
      Port:=edtPort.Value;
      Username:=hcbUser.Text;
      Password:=edtFtpPwd.Text;
      Directory:=hcbDir.Text;
      SecureMode:=FSecureMode;
      Passive:=cbPassive.Checked;
      CaseMode:=TTextChange(cbxCase.ItemIndex);
      if not cbProxy.Checked then FProxyPar.Mode:=fpcmNone;
      Proxy:=FProxyPar;
      Result:=true;
      end
    else Result:=false;
    end;
  end;

end.
