(* Delphi Dialog
   Hinweisfenster fr Status mit Fortschrittsanzeige (opt.)
   ========================================================

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Okt. 2002 
   *)

unit ProgWind;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Messages, Buttons, ExtCtrls, Prgrss, ComCtrls;

const
  Rand = 5;
  Breite = 271;

  wmCancelAction = WM_USER+1;

type
  TProgressWindow = class(TForm)
    StatusLabel: TLabel;
    ProgressBar: TProgressBar;
    Button: TBitBtn;
    ActionLabel: TLabel;
    procedure ButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FHandle : HWND; // Handle of calling form
    procedure WritePosition (Value : integer);
    procedure SetStatus (AStat : string);
    procedure SetError (AStat : string);
    procedure SetAction (AAction : string);
  public
    { Public declarations }
    WStatus,
    Stopped : boolean;
    property Position : integer write WritePosition;
    property Action : string write SetAction;
    property Error : string write SetError;
    property Status : string write SetStatus;

    procedure ShowWindow (CfHandle : HWND; ATitle,AAction : string; AWidth,AMin,AMax : integer);
    procedure CloseWindow;
  end;

var
  ProgressWindow: TProgressWindow;

implementation

{$R *.DFM}
{$R *.RES}

uses FileCtrl, GnuGetText;

procedure TProgressWindow.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  FHandle:=0;
  Stopped:=false; WStatus:=false;
  end;

{ ---------------------------------------------------------------- }
procedure TProgressWindow.WritePosition (Value : integer);
begin
  with ProgressBar do begin
    if Value<Min then Value:=Min;
    if Value>Max then Value:=Max;
    Position:=Value;
    end;
  end;

procedure TProgressWindow.SetStatus (AStat : string);
begin
  with StatusLabel do begin
    Font.Color:=clBlue;
    Caption:=MinimizeName(AStat,Canvas,ProgressBar.Width);
    end;
  end;

procedure TProgressWindow.SetError(AStat : string);
begin
  with StatusLabel do begin
    Font.Color:=clRed;
    Caption:=AStat;
    end;
  end;

procedure TProgressWindow.SetAction (AAction: string);
begin
  with ActionLabel do begin
    Caption:=AAction;
    end;
  end;

procedure TProgressWindow.ButtonClick(Sender: TObject);
begin
  Stopped:=true;
  if WStatus then Close
  else if FHandle<>0 then PostMessage (FHandle,wmCancelAction,0,0);
  end;

{ ---------------------------------------------------------------- }
(* Fortschrittsbalken anzeigen *)
procedure TProgressWindow.ShowWindow (CfHandle : HWND;
                                      ATitle,AAction   : string;
                                      AWidth,AMin,AMax : integer);
(* ATitle  : Fensterberschrift
   AAction : Name der Aktion
   AWidth  : Fensterbreite;
   Amin,Amax  : Randwerte des Balkens *)
var
  w : integer;
begin
  Stopped:=false; WStatus:=false;
  FHandle:=CfHandle;
  if AWidth>Width then Width:=AWidth;
  w:=ClientWidth;
  Caption:=ATitle;
  Action:=MinimizeName(AAction,Canvas,ClientWidth-2*ActionLabel.Left);
  Status:='';
  with Button do Left:=(w-Width) div 2;
  with ProgressBar do begin
    Width:=w-10;
    Min:=AMin; Max:=AMax;
    Position:=0;
    end;
  with Button do begin
    Glyph.LoadFromResourceName(HInstance,'CANCEL');
    Left:=(w-Width) div 2;
    Caption:=dgettext('dialogs','Cancel');
    end;
  Show;
  end;

procedure TProgressWindow.CloseWindow;
begin
  if Visible then Hide;
  if stopped then Action:=dgettext('dialogs','Canceled by user');
  WStatus:=true;
  with Button do begin
    Glyph.LoadFromResourceName(HInstance,'OK');
    Caption:=dgettext('dialogs','Ok');
    end;
  messagebeep (mb_OK);
  Application.BringToFront;
  ShowModal;
  end;

end.
