(* Delphi 7 Unit
   verschiedene Unterroutinen fr Windows Desktop und Shell

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Sep. 2002
     letzte nderung: Apr. 2006
   *)

unit WinShell;

interface

uses Windows, SysUtils, ShlObj, Classes;

const
{ Werte fr nFolder in SHGetSpecialFolderLocation
  # = fr Windows 9x, NT und 2000
  * = vermutlich nur Windows 98 und 2000
  + = vermutlich nur Windows NT und 2000
  ? = vermutlich nur Windows 2000 }
{ Deklarationen: siehe ShlObj
  CSIDL_DESKTOP                       = $0000;    (* # User\Desktop *)
  CSIDL_PROGRAMS                      = $0002;    (* # User\Startmen\Programme *)
  CSIDL_CONTROLS                      = $0003;
  CSIDL_PRINTERS                      = $0004;
  CSIDL_PERSONAL                      = $0005;    (* # pers. Verz. *)
  CSIDL_FAVORITES                     = $0006;    (* # User\Favoriten *)
  CSIDL_STARTUP                       = $0007;    (* # User\Startmen\Programme\Autostart *)
  CSIDL_RECENT                        = $0008;    (* # User\Recent *)
  CSIDL_SENDTO                        = $0009;    (* # User\SendTo *)
  CSIDL_BITBUCKET                     = $000A;
  CSIDL_STARTMENU                     = $000B;    (* # User\Startmen *)
  CSIDL_DESKTOPDIRECTORY              = $0010;    (* # User\Desktop *)
  CSIDL_DRIVES	                      = $0011;
  CSIDL_NETWORK                       = $0012;
  CSIDL_NETHOOD                       = $0013;    (* # User\Netzwerkumgebung *)
  CSIDL_FONTS                         = $0014;    (* # Windows\Fonts *)
  CSIDL_TEMPLATES                     = $0015;    (* # User\Vorlagen *)
  CSIDL_COMMON_STARTMENU              = $0016;    (* + All Users\Startmen *)
  CSIDL_COMMON_PROGRAMS               = $0017;    (* + All Users\Programme *)
  CSIDL_COMMON_STARTUP                = $0018;    (* + All Users\Autostart *)
  CSIDL_COMMON_DESKTOPDIRECTORY       = $0019;    (* + All Users\Desktop *)
  CSIDL_APPDATA                       = $001A;    (* # User\Anwendungsdaten *)
  CSIDL_PRINTHOOD                     = $001B;    (* # User\Druckumgebung *)
  CSIDL_LOCALAPPDATA                  = $001C;    (* * User\Lokale Einstellungen\Anwendungsdaten *)
  CSIDL_COMMON_FAVORITES              = $001F;    (* ? All Users\Favoriten *)
  CSIDL_CACHE                         = $0020;    (* + User\Lokale Einstellungen\Temporary Internet Files *)
  CSIDL_COOKIES                       = $0021;    (* + User\Cookies *)
  CSIDL_HISTORY                       = $0022;    (* + User\Lokale Einstellungen\Verlauf *)
  CSIDL_COMMON_APPDATA                = $0023;    (* ? All Users\Anwendungsdaten *)
  CSIDL_WINDIR	                      = $0024;    (* ? Windows-Verzeichnis *)
  CSIDL_SYSTEMDIR                     = $0025;    (* ? Windows-Systemverzeichnis *)
  CSIDL_MYPICTURES                    = $0027;    (* ? pers. Verz.\Eigene Bilder *)
  CSIDL_USERPROFILE                   = $0028;    (* ? User *)
  CSIDL_SHAREFILESDIR                 = $002B;    (* ? Programm-Verzeichnis *)
  CSIDL_COMMON_TEMPLATES              = $002D;    (* ? All Users\Vorlagen *)
  }
// neue Parameter aus ShFolder.h  (SDK fr Win2003), andere Werte siehe ShlObj.pas
  CSIDL_LOCAL_APPDATA             = $001C;     // non roaming, user\Local Settings\Application Data
  CSIDL_COMMON_APPDATA            = $0023;     // All Users\Application Data
  CSIDL_WINDOWS                   = $0024;     // GetWindowsDirectory()
  CSIDL_SYSTEM                    = $0025;     // GetSystemDirectory()
  CSIDL_PROGRAM_FILES             = $0026;     // C:\Program Files
  CSIDL_MYPICTURES                = $0027;     // My Pictures, new for Win2K
  CSIDL_PROGRAM_FILES_COMMON      = $002b;     // C:\Program Files\Common
  CSIDL_COMMON_DOCUMENTS          = $002e;     // All Users\Documents
  CSIDL_RESOURCES                 = $0038;     // %windir%\Resources\, For theme and other windows resources.
  CSIDL_RESOURCES_LOCALIZED       = $0039;     // %windir%\Resources\<LangID>, for theme and other windows specific resources.


  {ulFlags in TBrowseInfo}
  {$EXTERNALSYM BIF_NEWDIALOGSTYLE}
  BIF_NEWDIALOGSTYLE          =   $0040;   // Use the new dialog layout with the ability to resize
  {$EXTERNALSYM BIF_NONEWFOLDERBUTTON}
  BIF_NONEWFOLDERBUTTON       =   $0200;   // Do not add the "New Folder" button to

{ ---------------------------------------------------------------- }
(* Verzeichnisse auf Desktop oder im Startmen erstellen *)
function NewDesktopFolder (Foldername : string;
                           Typ        : integer): boolean;

(* Verzeichnisnamen aus Desktop oder Startmen *)
function GetDesktopFolder (Typ : integer): string;

procedure RefreshDesktop;

{ Dialog Verzeichnisauswahl }
function GetDirectory (Title      : string;
                       RootTyp    : integer;
                       var Dir    : string;
                       var ImgNdx : integer) : boolean;

procedure MakeLink(const LinkObj,ProgName,Arg,WorkDir,Desc: string);
function GetLink (const LinkObj : string; var ProgName,Arg,WorkDir,Desc: string) : Boolean;

{ ---------------------------------------------------------------- }
(* Icon in Taskbar aufnehmen *)
procedure TaskBarAddIcon (WinHandle  : HWnd;
                          MessageID  : integer;
                          IconHandle : HIcon;
                          Hint       : string);
(* Icon in Taskbar ndern *)
procedure TaskBarChangeIcon (WinHandle  : HWnd;
                             IconHandle : HIcon);
(* Icon aus Taskbar entfernen *)
procedure TaskBarRemoveIcon (WinHandle  : HWnd);

{ ---------------------------------------------------------------- }
// Shell file operations
function ShellCopyFiles (WinHandle        : HWnd;
                         Source,Dest,Hint : string;
                         Silent           : boolean) : boolean;

function ShellMoveFiles (WinHandle        : HWnd;
                         Source,Dest,Hint : string;
                         Silent           : boolean) : boolean;

function ShellDeleteFiles (WinHandle   : HWnd;
                           Source,Hint : string;
                           Recycle     : boolean) : boolean;

function ShellDeleteAll (WinHandle   : HWnd;
                         Source,Hint : string;
                         Recycle     : boolean) : boolean;

{ ---------------------------------------------------------------- }
implementation

uses ActiveX, ComObj, Shellapi;

{ ------------------------------------------------------------------- }
{ ---------------------------------------------------------------- }
(* Windows-Desktop *)
(* Verzeichnisse auf Desktop oder im Startmen erstellen *)
function NewDesktopFolder (Foldername : string;
                           Typ        : integer): boolean;
var
  pidl          : PItemIDList;
  ProgramPath   : PChar;
begin
  result := False;
  if SUCCEEDED(SHGetSpecialFolderLocation(0,Typ,pidl)) then begin
    ProgramPath := StrAlloc(max_path);
    SHGetPathFromIDList(pidl, ProgramPath);
    SetLastError(0);
    CreateDirectory(PChar(ProgramPath + '\\' + Foldername), nil );
    if (GetLastError()=0) or (GetLastError()=ERROR_ALREADY_EXISTS) then result := True;
    StrDispose(ProgramPath);
    end;
  end;

(* Verzeichnisnamen aus Desktop oder Startmen *)
function GetDesktopFolder (Typ : integer): string;
var
  pidl          : PItemIDList;
  FolderPath    : PChar;
  pMalloc       : IMalloc;
begin
  pidl:=nil;
  SHGetMalloc(pMalloc);
  if SUCCEEDED(SHGetSpecialFolderLocation(0,Typ,pidl)) then begin
    FolderPath := StrAlloc(max_path);
    SHGetPathFromIDList(pidl,FolderPath);
    SetLastError(0);
    Result:=FolderPath;
    end
  else Result:='';
  if pidl<>nil then pMalloc.Free(pidl);
  pMalloc._Release;
  end;

procedure RefreshDesktop;
var
  pidl          : PItemIDList;
  pMalloc       : IMalloc;
begin
  pidl:=nil;
  SHGetMalloc(pMalloc);
  SHGetSpecialFolderLocation(0,CSIDL_DESKTOP,pidl);
  SHChangeNotify(SHCNE_ALLEVENTS,SHCNF_IDLIST,pidl,nil);
  if pidl<>nil then pMalloc.Free(pidl);
  pMalloc._Release;
  end;

{ ---------------------------------------------------------------- }
{ Dialog Verzeichnisauswahl }
function GetDirectory (Title      : string;
                       RootTyp    : integer;
                       var Dir    : string;
                       var ImgNdx : integer) : boolean;
var
  bi            : TBROWSEINFO;
  lpBuffer      : PChar;
  pidlPrograms,
  pidlBrowse    : PItemIDList;
  ok            : boolean;
  pMalloc       : IMalloc;
begin
  pidlPrograms:=nil; pidlBrowse:=nil;
  SHGetMalloc(pMalloc);
  Result:=false;
  ok:=(SUCCEEDED(SHGetSpecialFolderLocation(getactivewindow,RootTyp,pidlPrograms)));
  if not ok then ok:=(SUCCEEDED(SHGetSpecialFolderLocation(getactivewindow,CSIDL_DESKTOP,pidlPrograms)));
  if ok then begin
   lpBuffer := StrAlloc(max_path);
    with bi do begin
      hwndOwner := getactivewindow;
      pidlRoot := pidlPrograms;
      pszDisplayName := lpBuffer;
      lpszTitle := pChar(Title);
      ulFlags := BIF_RETURNONLYFSDIRS or BIF_NEWDIALOGSTYLE;
      lpfn := nil;
      lParam := 0;
      end;
    pidlBrowse := SHBrowseForFolder(bi);
    if (pidlBrowse <> nil) then begin
      if (SHGetPathFromIDList(pidlBrowse,lpBuffer)) then begin
        Dir:=lpBuffer;
        ImgNdx:=bi.iImage;
        Result:=true;
        end;
      end;
    StrDispose(lpBuffer);
    end;
  if pidlPrograms<>nil then pMalloc.Free(pidlPrograms);
  if pidlBrowse<>nil then pMalloc.Free(pidlBrowse);
  pMalloc._Release;
  end;


{ ---------------------------------------------------------------- }
procedure MakeLink(const LinkObj,ProgName,Arg,WorkDir,Desc: string);
var
  puk : IUnknown;
  psl : IShellLink;
  ppf : IPersistFile;
  ws  : WideString;
  rem : boolean;
begin
  puk:=CreateComObject(CLSID_ShellLink);
  psl:=puk as IShellLink;
  ppf:=puk as IPersistFile;
  with psl do begin
    SetPath(pChar(ProgName));
    SetArguments(PChar(Arg));
    if length(ProgName)>0 then SetIconLocation(PChar(ProgName),0);
    if length(WorkDir)>0 then SetWorkingDirectory(PChar(WorkDir));
    if length(Desc)>0 then SetDescription(PChar(Desc));
    end;
  rem:=not FileExists(LinkObj);
  ws:=LinkObj;
  ppf.Save(PWChar(ws),rem);
  end;

function GetLink (const LinkObj : string; var ProgName,Arg,WorkDir,Desc: string) : Boolean;
var
  puk : IUnknown;
  psl : IShellLink;
  ppf : IPersistFile;
  ws  : WideString;
  sb  : PChar;
  pfd : TWin32FindData;
begin
  Result:=false;
  if FileExists(LinkObj) then begin
    puk:=CreateComObject(CLSID_ShellLink);
    psl:=puk as IShellLink;
    ppf:=puk as IPersistFile;
    ws:=LinkObj; sb:=StrAlloc(max_path);
    ppf.Load(PWChar(ws),STGM_READ);
    with psl do if succeeded(GetPath(sb,MAX_PATH,pfd,SLGP_UNCPRIORITY)) then begin
      ProgName:=sb;
      if succeeded(GetArguments(sb,MAX_PATH)) then Arg:=sb;
      if succeeded(GetWorkingDirectory(sb,MAX_PATH)) then WorkDir:=sb;
      if succeeded(GetDescription(sb,MAX_PATH)) then Desc:=sb;
      Result:=true;
      end;
    StrDispose(sb);
    puk:=nil;
    end
  end;

{ ---------------------------------------------------------------- }
(* Windows-Taskbar *)
(* Icon in Taskleiste aufnehmen *)
procedure TaskBarAddIcon (WinHandle  : HWnd;
                          MessageID  : integer;
                          IconHandle : HIcon;
                          Hint       : string);
var
  tnid : TNOTIFYICONDATA;
begin
  with tnid do begin
    cbSize := sizeof(TNOTIFYICONDATA); // Grenangabe der Struktur
    Wnd := WinHandle;                  // Handle des Message-Empfngers
    uID := 1;                          // ID beliebig
    uFlags := 0;                       // siehe Tabelle
    if MessageID>0 then uFlags:=NIF_MESSAGE;
    uCallbackMessage := MessageID;     // Message Identifier
    if Assigned(pointer(IconHandle)) then uFlags:=uFlags or NIF_ICON;
    hIcon := Iconhandle;               // Iconhandle
    if length(Hint)>0 then uFlags:=uFlags or NIF_TIP;
    strpcopy(szTip,pchar(Hint));        // Tooltiptext
    end;
  Shell_NotifyIcon(NIM_ADD, @tnid);       // Registrieren ...
  end;

(* Icon in Taskbar ndern *)
procedure TaskBarChangeIcon (WinHandle  : HWnd;
                             IconHandle : HIcon);
var
  tnid : TNOTIFYICONDATA ;
begin
  with tnid do begin
    cbSize := sizeof(TNOTIFYICONDATA);
    Wnd := WinHandle;
    uID := 1;
    uFlags := 0;                       // siehe Tabelle
    if Assigned(pointer(IconHandle)) then uFlags:=uFlags or NIF_ICON;
    hIcon := Iconhandle;               // Iconhandle
    end;
  Shell_NotifyIcon(NIM_MODIFY, @tnid);
  end;

(* Icon aus Taskbar entfernen *)
procedure TaskBarRemoveIcon (WinHandle  : HWnd);
var
  tnid : TNOTIFYICONDATA ;
begin
  tnid.cbSize := sizeof(TNOTIFYICONDATA);
  tnid.Wnd := WinHandle;
  tnid.uID := 1;
  Shell_NotifyIcon(NIM_DELETE, @tnid);
  end;

{ ---------------------------------------------------------------- }
// Shell file operations
const
  // nachfolgende Definition fehlen in ShellApi.pas
  FOF_NOERRORUI             = $0400;  // don't display a user interface if an error occurs
  FOF_NOCOPYSECURITYATTRIBS = $0800;  // don't copy NT file Security Attributes
  FOF_NORECURSION           = $1000;  // don't recurse into directories.
  FOF_NO_CONNECTED_ELEMENTS = $2000;  // don't operate on connected elements.
  FOF_WANTNUKEWARNING       = $4000;  // during delete operation, warn if nuking instead of recycling (partially overrides FOF_NOCONFIRMATION)
  FOF_NORECURSEREPARSE      = $8000;  // treat reparse points as objects, not containers

function DoFileOperation(WinHandle  : HWnd; Source,Dest,Caption : string; Action,Flags : integer) : boolean;
var
  FileOpStruct : TSHFileOpStruct;
begin
  with FileOpStruct do begin
    wnd:=WinHandle;
    wFunc:=Action;
    pFrom:=PChar(Source+#0);
    if length(Dest)>0 then pTo:=PChar(Dest+#0)
    else pTo:=nil;
    lpszProgressTitle:=pchar(Caption);
    fFlags:=Flags;
    end;
  Result:=SHFileOperation(FileOpStruct)=0;
  end;

function ShellCopyFiles (WinHandle : HWnd; Source,Dest,Hint : string; Silent : boolean) : boolean;
var
  f : integer;
begin
  if Silent then f:=FOF_SILENT else f:=0;
  Result:=DoFileOperation(WinHandle,Source,Dest,Hint,FO_COPY,
    f+FOF_NOERRORUI+FOF_FILESONLY+FOF_MULTIDESTFILES+FOF_NOCONFIRMATION+FOF_NOCONFIRMMKDIR+FOF_NO_CONNECTED_ELEMENTS);
  end;

function ShellMoveFiles (WinHandle        : HWnd;
                         Source,Dest,Hint : string;
                         Silent           : boolean) : boolean;
var
  f : integer;
begin
  if Silent then f:=FOF_SILENT else f:=0;
  Result:=DoFileOperation(WinHandle,Source,Dest,Hint,FO_MOVE,
    f+FOF_NOERRORUI+FOF_FILESONLY+FOF_MULTIDESTFILES+FOF_NOCONFIRMATION+FOF_NOCONFIRMMKDIR+FOF_NO_CONNECTED_ELEMENTS);
  end;

function ShellDeleteFiles (WinHandle   : HWnd;
                           Source,Hint : string;
                           Recycle     : boolean) : boolean;
var
  f : integer;
begin
  if Recycle then f:=FOF_ALLOWUNDO else f:=0;
  Result:=DoFileOperation(WinHandle,Source,'',Hint,FO_DELETE,
    f+FOF_FILESONLY+FOF_MULTIDESTFILES+FOF_NOCONFIRMATION+FOF_NO_CONNECTED_ELEMENTS);
  end;

function ShellDeleteAll (WinHandle   : HWnd;
                         Source,Hint : string;
                         Recycle     : boolean) : boolean;
var
  f : integer;
begin
  if Recycle then f:=FOF_ALLOWUNDO else f:=0;
  Result:=DoFileOperation(WinHandle,Source,'',Hint,FO_DELETE,
    f+FOF_MULTIDESTFILES+FOF_NOCONFIRMATION+FOF_NO_CONNECTED_ELEMENTS);
  end;


end.
